#!/bin/sh

SYSLOG_FILE=/var/log/syslog
TEMP_FILE=/tmp/sysklogd_tmplog
TMP_BLOCK_DATA=/tmp/tmp_block_data
TEST_NAME=""
which_syslogd="`which syslogd`"
which_klogd="`which klogd`"
SECOND=1
SYSTEM=""

# Additional function to print results and exit
# $1 - test result
print_test_results() 
{	
	echo "$TEST_NAME: "	
	echo "$TEST_NAME: *****************"
	echo "$TEST_NAME: Test case results "
	if [ $1 -eq 0 ] ; then
		echo "$TEST_NAME:     PASS"
	else
		echo "$TEST_NAME:     FAIL"
	fi
	echo "$TEST_NAME: *****************"
	echo "$TEST_NAME: "	
	return $1
}

# Additional function to test correct execution of input command
assert_cmd() 
{
	echo "$TEST_NAME: Launching $*"
	eval $*
	if [ $? -ne 0 ] ; then 
		echo "$TEST_NAME: Error occures during $*"
		restore_logd
		print_test_results 1
		exit 1
	fi
}

# Additional function for comparing results
assert_condition() 
{
	echo "$TEST_NAME: Comparing $1"
	if [ `echo $1` ] ; then 
		echo "$TEST_NAME: Comparing of $1 returns true"
		return 0
	else
		echo "$TEST_NAME: $2"
		restore_logd
		print_test_results 1
		exit 1
	fi
}
# Additional function for comparing strings
assert_str()
{
	echo "$TEST_NAME: Comparing $1 $2 $3"
	if [ "`echo $1`" $2 "`echo $3`" ] ; then 
		echo "$TEST_NAME: Comparing of $1 with $3 by condition $2 returns true"
		return 0
	else
		echo "$TEST_NAME: $4"
		restore_logd
		print_test_results 1
		exit 1
	fi
}

# Additional function. Write unique string into syslog for test purpose
# $1 shows what string it showld be: {START, STOP}
write_string() 
{
	STRING="Add line from syslogd for test purpose: $1 `date`"
	STRING=`echo $STRING`
	echo $STRING >> $SYSLOG_FILE
	echo "$TEST_NAME: $STRING"

	case $1 in
	START )  
		STARTSTRING=$STRING ;;
	STOP )
		STOPSTRING=$STRING	;;	
    * )
		echo "$TEST_NAME: Function write_string() got unknown parameter.."
		exit 1;;
	esac 
	delay 1
}


# Additional function to get data block from syslog between STARTSTRING and STOPSTRING
get_block_data_from_syslog()
{
	echo "$TEST_NAME: Getting data block from $SYSLOG_FILE to analyze"
	rm -f $TMP_BLOCK_DATA
	assert_cmd 'sed -n "/$STARTSTRING/,/$STOPSTRING/p" $SYSLOG_FILE  > $TMP_BLOCK_DATA'
	echo "$TEST_NAME: File $TMP_BLOCK_DATA contains"
	cat $TMP_BLOCK_DATA
	echo "$TEST_NAME:"
}

# Print test case header
print_test_header() 
{
	echo "$TEST_NAME: *****************"
	echo "$TEST_NAME: Start test case   "
	echo "$TEST_NAME: *****************"
}

# Restore $1 daemon
restore_logd()
{
	echo " "
	echo "$TEST_NAME: Restoring $SYSTEM in usual mode"
	if [ "$SYSTEM" = "klogd" ] ; then 
		start_klogd 
	else
		start_syslogd $NORMAL
	fi	
	rm -f $TMP_BLOCK_DATA
}

# Delay function
# $1 seconds to delay 
delay() 
{
	echo "$TEST_NAME: Do sleep $1"
	sleep $1
}
