#!/bin/sh
. /usr/lib/ci-tests/sysklogd/sysklogd_add_functions.sh
MAXMARKLINECOUNT=5

SYSLOG_CONF=/etc/syslog.conf
TEST_CONF_FILE=/usr/share/sysklogd-tests/syslog_test.conf
TEST_CONF_FILE_ALL_LEVELS=/usr/share/sysklogd-tests/syslog_test_all_levels.conf
TMP_CONF_FILE=/tmp/tmp_conf_file.conf

MARK_SEARCH="[-][-] MARK [-][-]"
NORMAL="normal"
DEBUG="debug"

# Start syslogd in one of two following modes: {normal, debug}, $2, $3 - parameter to start it
start_syslogd() 
{
	echo "$TEST_NAME: start_syslogd ($*) "
	if [ "$1" != "$NORMAL" ] && [ "$1" != "$DEBUG" ] ; then
		echo "$TEST_NAME: Routine strat_syslog() was called with wrong parameter"
		return 1
	fi

	if [ `pidof syslogd` ] ; then 
		kill_syslogd
	fi
	
	echo "$TEST_NAME: Starting syslog in $1 mode"
	rm -f $TEMP_FILE
	if [ $1 = "$NORMAL" ] ; then
		assert_cmd "$which_syslogd $2 $3"
	else
		assert_cmd "$which_syslogd -d > $TEMP_FILE & "
	fi
	echo "$TEST_NAME: syslogd was started succesfully"
}

kill_syslogd()
{
	echo "$TEST_NAME: killing syslogd"
	assert_cmd "kill -KILL `pidof syslogd`"
	delay $SECOND
}

#########################################################
# 
# Test signal's influence on syslogd
#
#########################################################
execute_syslog_tst()
{	
	echo "$TEST_NAME: In execute_syslog_tst($*)"

	write_string "START"
	assert_cmd $testbin
	write_string "STOP"

	assert_cmd get_block_data_from_syslog

	echo "$TEST_NAME: File $TMP_BLOCK_DATA contains"
	cat $TMP_BLOCK_DATA
	echo ""

	assert_condition "`grep 'DOTEST' $TMP_BLOCK_DATA -c` -eq $1"
	assert_condition "`grep 'DOTEST*.*Debug' $TMP_BLOCK_DATA -c` -eq $2"  
	
	echo "$TEST_NAME: execute_syslog_tst() finished OK"
}
#Test case 1
test_SIGHUP_sig()
{
	echo "$TEST_NAME: Preparing syslog.conf file based on test conf file "
	assert_cmd "cat $TEST_CONF_FILE > $TMP_CONF_FILE"
	start_syslogd $NORMAL -f $TMP_CONF_FILE
	assert_cmd "execute_syslog_tst 7 0"
	delay $SECOND	

	echo "$TEST_NAME: Preparing syslog.conf file based on usual syslog conf file "
	echo "$TEST_NAME: Begin of $SYSLOG_CONF file"
	cat $SYSLOG_CONF
	echo "$TEST_NAME: End of $SYSLOG_CONF file"

	assert_cmd "cat $TEST_CONF_FILE_ALL_LEVELS > $TMP_CONF_FILE "
	assert_cmd "kill -SIGHUP `cat /var/run/syslogd.pid`"

	echo "$TEST_NAME: Begin of $TMP_CONF_FILE file"
	cat $TMP_CONF_FILE
	echo "$TEST_NAME: End of $TMP_CONF_FILE file"

	assert_cmd "execute_syslog_tst 8 1"

	echo "$TEST_NAME: Syslogd received SIGHUP signal and reinitialyze itself. OK"
	rm -f $TMP_CONF_FILE
	restore_logd 
	return 0		
}


# Test case #2
test_SIGTERM_sig()
{
	start_syslogd $NORMAL
	assert_cmd	"kill -SIGTERM `cat /var/run/syslogd.pid`"
	echo "$TEST_NAME: Signal is caught. Terminating ..."
	delay 1

	assert_condition " ! `pidof syslogd` "  "$TEST_NAME: Daemon still works, although it shouldn't"

	echo "$TEST_NAME: Daemon work was terminated"
	restore_logd 
}

# Test case #3
test_SIGUSR1_sig()
{
	start_syslogd $DEBUG
	assert_cmd	"kill -SIGUSR1 `pidof syslogd`"
	echo "$TEST_NAME: Signal is caught. Start printing debug info ... "
	delay 2

	assert_condition " `grep "Calling select, active file descriptors" $TEMP_FILE -c ` -gt 0" "$TEST_NAME: Signal SIGUSR1 isn't caught"

	echo "$TEST_NAME: Signal SIGUSR1 is caught"
	restore_logd 
}

send_sig() # $1 - signal which it requires to send, $2 - daemon mode 
{
	SIG=$1
	start_syslogd $2
	assert_cmd	"kill -SIGUSR1 `pidof syslogd`"
	assert_cmd "kill -$SIG `pidof syslogd`"
	delay 2
	
	if [ "$2" =  "$DEBUG" ] ; then
		assert_condition "`pidof syslogd`" "Daemon syslogd is exiting on signal $SIG, but should ignore it "	
		echo "$TEST_NAME: Syslogd in normal mode is exiting on siganl $SIG as expected"
	else
		assert_condition "! `pidof syslogd`" "Signal $SIG is ignored by syslogd in normal mode, but it should not"
		echo "$TEST_NAME: Signal $SIG is ignored by syslogd in debug mode as expected."
	fi
	restore_logd 
}

test_SIGINT_sig()
{
	send_sig  "SIGINT" $DEBUG
	send_sig  "SIGINT" $NORMAL
}

test_SIGQUIT_sig()
{
	send_sig  "SIGQUIT" $DEBUG
	send_sig  "SIGQUIT" $NORMAL 
}
#########################################################
# Test case: option -n
# Do not fork
test_option_n()
{
	echo "$TEST_NAME: test_option_n () "
	start_syslogd $NORMAL -n "&"
	Pid=`pidof syslogd`
	PPid=`cat /proc/$Pid/status  | grep PPid: | awk '{print $2}'`

	echo "$TEST_NAME: Pid = $Pid, PPid = $PPid "

	assert_condition "$PPid -ne 1" "$TEST_NAME: Syslogd makes forking and becomes a daemon "
	echo "$TEST_NAME: Syslogd avoids forking"

	restore_logd
}

#Test case: option -f config file 
test_option_f()
{
	start_syslogd $NORMAL -f $TEST_CONF_FILE
	assert_cmd "execute_syslog_tst 7 0"
	echo "$TEST_NAME: Option \"-f config_file\" works"
	restore_logd 
}

# Test case: option -m interval
test_option_m()
{
	write_string "START"
	start_syslogd $NORMAL -m 1
	delay 306
	write_string "STOP"
	assert_cmd get_block_data_from_syslog 
	rescount=`grep "$MARK_SEARCH" -c $TMP_BLOCK_DATA`

	assert_condition "$rescount -le $MAXMARKLINECOUNT" "$TEST_NAME: Option \"-m interval\" doesn't work"
	assert_condition "$rescount -gt 0" "$TEST_NAME: Option \"-m interval\" doesn't work"

	echo "$TEST_NAME: Option \"-m interval\" works"
	restore_logd 
}

#########################################################
# 
# Start main programm
#
#########################################################

print_usage ()
{
	echo "Usage: $0 -s SIG_NAME [testbin] runs syslogd test for input SIG_NAME;" 
	echo "       Possible signals {SIGUSR1, SIGHUP, SIGINT, SIGTERM, SIGQUIT} "
	echo "       testbin - program which writes into syslog messages with diff log level"
	echo "Usage: $0 -o option runs syslogd test for input option."
	echo "       Possible options {f, m, n} "
	exit 1
}

if [ $# -le 1 ] ; then 
	print_usage
fi
SYSTEM="syslogd"
while getopts "s:o:" opt; do
	case $opt in
		s )
			case $OPTARG in	
				"SIGINT" | "SIGTERM" | "SIGUSR1" | "SIGQUIT" )
					TEST_NAME="test_"$OPTARG"_sig"
					print_test_header  
					assert_cmd $TEST_NAME 
					print_test_results 0 ;;
				"SIGHUP")
					if [ ! $3 ] ; 
					then
						echo "TEST: To test $OPTARG signal it requires to define test programm"
						print_usage
					fi
					testbin=$3
					TEST_NAME="test_"$OPTARG"_sig"
					print_test_header  
					assert_cmd $TEST_NAME 
					print_test_results 0 ;;

				*)	
					print_usage ;;
			esac ;;
        o )
			case $OPTARG in	
 				"v" | "m" | "n" )
					TEST_NAME="test_option_$OPTARG" 
					print_test_header  
					assert_cmd $TEST_NAME 
					print_test_results 0 ;;

				"f")
					if [ ! $3 ] ; 
					then
						echo "TEST: To test -f option it requires to define test programm"
						print_usage
					fi
					testbin=$3
					TEST_NAME="test_option_$OPTARG" 
					print_test_header  
					assert_cmd $TEST_NAME 
					print_test_results 0 ;;

				*)	
					print_usage;;
			esac ;;
        * ) print_usage ;;
   esac
done

# End of test
